// Imports
import { Icon } from "@chakra-ui/react";
import { MdHome } from "react-icons/md";
import { IoBriefcaseSharp, IoCloudUpload, IoCloudDownload } from "react-icons/io5";
import { IoTimeSharp } from "react-icons/io5";
import { GiRibcage } from "react-icons/gi";
import { IoMdCog } from "react-icons/io";
import {
  FaUsers,
  FaUserShield,
  FaUserCheck,
  FaMapMarkedAlt,
  FaHistory,
  FaListUl,
} from "react-icons/fa";
import {
  BsPersonFillGear,
  BsPersonBadge,
  BsFillPeopleFill,
} from "react-icons/bs";
import { GiProgression } from "react-icons/gi";
import { useSession } from "next-auth/react";
import { hasPermission } from "./utils/navigation";
import { IRoute } from "@/types/navigation";
import { CalendarIcon } from "@chakra-ui/icons";
import { FaCarTunnel } from "react-icons/fa6";
import {
  TbCheck,
  TbChecklist,
  TbCheckupList,
  TbHourglass,
  TbTransfer,
} from "react-icons/tb";
import { HiOutlineDocumentReport } from "react-icons/hi";

const routes = (role: string): IRoute[] => {
  const { data: session } = useSession();
  const permissions = session?.user?.permissions ?? [];

  const adminRoutes: IRoute[] = [
    {
      name: "Dashboard",
      layout: "/admin",
      path: "/dashboard",
      icon: <Icon as={MdHome} width="20px" height="20px" color="inherit" />,
    },
    {
      name: "Attendance",
      layout: "/admin",
      path: "/attendance",
      icon: <Icon as={CalendarIcon} width="20px" height="20px" color="inherit" />,
      permissionPrefix: "attendance",
    },
    {
      name: "Cases",
      layout: "/admin",
      path: "/cases",
      icon: <Icon as={IoBriefcaseSharp} width="20px" height="20px" color="inherit" />,
      permissionPrefix: "case",
    },
    {
      name: "Roles",
      layout: "/admin",
      path: "/roles",
      icon: <Icon as={FaUserShield} width="20px" height="20px" color="inherit" />,
      permissionPrefix: "role",
    },
    {
      name: "Users",
      layout: "/admin",
      path: "/users",
      icon: <Icon as={FaUsers} width="20px" height="20px" color="inherit" />,
      permissionPrefix: "user",
    },
    {
      name: "Assignments",
      layout: "/admin",
      path: "",
      icon: <Icon as={FaUsers} width="20px" height="20px" color="inherit" />,
      permissionPrefix: "user",
      children: [
        {
          name: "List",
          layout: "/admin",
          path: "/assignments",
          icon: <Icon as={FaListUl} width="20px" height="20px" color="inherit" />,
          permissionPrefix: "assignment",
        },
        {
          name: "Transfer",
          layout: "/admin",
          path: "/assignments/transfer",
          icon: <Icon as={TbTransfer} width="20px" height="20px" color="inherit" />,
          permissionPrefix: "batch",
        },
      ],
    },
    {
      name: "Batch",
      layout: "/admin",
      path: "/batches",
      icon: <Icon as={IoCloudUpload} width="20px" height="20px" color="inherit" />,
      permissionPrefix: "batch",
    },
    {
      name: "Reports",
      layout: "/admin",
      path: "/reports",
      icon: <Icon as={HiOutlineDocumentReport} width="20px" height="20px" color="inherit" />,
      permissionPrefix: "",
    },
    {
      name: "Attempt",
      layout: "/admin",
      path: "",
      icon: <Icon as={FaUserCheck} width="20px" height="20px" color="inherit" />,
      permissionPrefix: "attempt",
      children: [
        {
          name: "KYC",
          layout: "/admin",
          path: "/attempt/kyc",
          icon: <Icon as={FaListUl} width="20px" height="20px" color="inherit" />,
          permissionPrefix: "kyc",
        },
        {
          name: "DC Attempt",
          layout: "/admin",
          path: "/attempt/dc-form",
          icon: <Icon as={BsPersonBadge} width="20px" height="20px" color="inherit" />,
          permissionPrefix: "dc-form",
        },
        {
          name: "RO Attempt",
          layout: "/admin",
          icon: <Icon as={BsFillPeopleFill} width="20px" height="20px" color="inherit" />,
          path: "/attempt/consumer",
          permissionPrefix: "consumer",
        },
      ],
    },
    {
      name: "Location",
      layout: "/admin",
      path: "",
      icon: <Icon as={FaMapMarkedAlt} width="20px" height="20px" color="inherit" />,
      permissionPrefix: "location",
      children: [
        {
          name: "Live Tracking",
          layout: "/admin",
          path: "/location/live",
          icon: <Icon as={GiProgression} width="20px" height="20px" color="inherit" />,
          permissionPrefix: "location",
        },
        {
          name: "History",
          layout: "/admin",
          icon: <Icon as={FaHistory} width="20px" height="20px" color="inherit" />,
          path: "/location/history",
          permissionPrefix: "location",
        },
      ],
    },
    {
      name: "Settings",
      layout: "/admin",
      path: "/settings",
      icon: <Icon as={IoMdCog} width="20px" height="20px" color="inherit" />,
      children: [
        {
          name: "Profile",
          layout: "/admin",
          icon: <Icon as={BsPersonFillGear} width="20px" height="20px" color="inherit" />,
          path: "/profile",
        },
        {
          name: "IBC",
          layout: "/admin",
          path: "/ibc",
          icon: <Icon as={GiRibcage} width="20px" height="20px" color="inherit" />,
          permissionPrefix: "setting.ibc",
        },
        {
          name: "Cycle Days",
          layout: "/admin",
          path: "/cycle-days",
          icon: <Icon as={IoTimeSharp} width="20px" height="20px" color="inherit" />,
          permissionPrefix: "setting.cycle-day",
        },
        {
          name: "Assignment Priority",
          layout: "/admin",
          path: "/assignment-priority",
          icon: <Icon as={IoTimeSharp} width="20px" height="20px" color="inherit" />,
          permissionPrefix: "setting.assignment-priority",
        },
        {
          name: "Project",
          layout: "/admin",
          path: "/project", // Path for project module
          icon: <Icon as={IoCloudUpload} width="20px" height="20px" color="inherit" />,
          permissionPrefix: "setting.project",
        },
        {
          name: "Download",
          layout: "/admin",
          path: "/download",
          icon: <Icon as={IoCloudDownload} width="20px" height="20px" color="inherit" />,
          permissionPrefix: "file",
        },
      ],
    },
  ];

  const officerRoutes: IRoute[] = [
    {
      name: "Dashboard",
      layout: "/officer",
      path: "/dashboard",
      icon: <Icon as={MdHome} width="20px" height="20px" color="inherit" />,
    },
    {
      name: "RO Attempt",
      layout: "/officer",
      path: "",
      icon: (
        <Icon as={FaCarTunnel} width="20px" height="20px" color="inherit" />
      ),
      permissionPrefix: "consumer",
      children: [
        {
          name: "Uploads",
          layout: "/officer",
          path: "/consumers",
          icon: (
            <Icon
              as={GiProgression}
              width="20px"
              height="20px"
              color="inherit"
            />
          ),
          permissionPrefix: "consumer",
        },
        {
          name: "Drafts",
          layout: "/officer",
          icon: (
            <Icon as={FaHistory} width="20px" height="20px" color="inherit" />
          ),
          path: "/consumer-drafts",
          permissionPrefix: "consumer",
        },
      ],
    },
    {
      name: "DC Attempt",
      layout: "/officer",
      path: "",
      icon: (
        <Icon as={TbChecklist} width="20px" height="20px" color="inherit" />
      ),
      permissionPrefix: "dc-form",
      children: [
        {
          name: "Uploads",
          layout: "/officer",
          path: "/dc",
          icon: (
            <Icon as={TbCheck} width="20px" height="20px" color="inherit" />
          ),
          permissionPrefix: "dc-form",
        },
        {
          name: "Drafts",
          layout: "/officer",
          path: "/dc-drafts",
          icon: (
            <Icon as={TbHourglass} width="20px" height="20px" color="inherit" />
          ),
          permissionPrefix: "dc-form",
        },
      ],
    },
    {
      name: "KYCs",
      layout: "/officer",
      path: "/",
      icon: <Icon as={FaListUl} width="20px" height="20px" color="inherit" />,
      permissionPrefix: "kyc",
      children: [
        {
          name: "Uploads",
          layout: "/officer",
          path: "/kycs",
          icon: (
            <Icon
              as={GiProgression}
              width="20px"
              height="20px"
              color="inherit"
            />
          ),
          permissionPrefix: "kyc",
        },
        {
          name: "Drafts",
          layout: "/officer",
          icon: (
            <Icon as={FaHistory} width="20px" height="20px" color="inherit" />
          ),
          path: "/kycs-drafts",
          permissionPrefix: "kyc",
        },
      ],
    },
    {
      name: "Assignments",
      layout: "/officer",
      path: "/assignments",
      icon: (
        <Icon as={TbCheckupList} width="20px" height="20px" color="inherit" />
      ),
    },
  ];

  const supervisorRoutes: IRoute[] = [
    {
      name: "Dashboard",
      layout: "/officer",
      path: "/dashboard",
      icon: <Icon as={MdHome} width="20px" height="20px" color="inherit" />,
    },
    {
      name: "Attendance",
      layout: "/officer",
      path: "/attendance",
      icon: (
        <Icon as={CalendarIcon} width="20px" height="20px" color="inherit" />
      ),
      permissionPrefix: "attendance",
    },
    {
      name: "Consumers",
      layout: "/officer",
      path: "/consumers",
      icon: (
        <Icon as={FaCarTunnel} width="20px" height="20px" color="inherit" />
      ),
    },
    {
      name: "Cases",
      layout: "/officer",
      path: "/cases",
      icon: (
        <Icon
          as={IoBriefcaseSharp}
          width="20px"
          height="20px"
          color="inherit"
        />
      ),
      permissionPrefix: "case",
    },
    {
      name: "Assignments",
      layout: "/officer",
      path: "/assignments",
      icon: (
        <Icon as={TbCheckupList} width="20px" height="20px" color="inherit" />
      ),
    },
    {
      name: "Batch",
      layout: "/officer",
      path: "/batches",
      icon: (
        <Icon as={IoCloudUpload} width="20px" height="20px" color="inherit" />
      ),
      permissionPrefix: "batch",
    },
    {
      name: "Attempt",
      layout: "/officer",
      path: "",
      icon: (
        <Icon as={FaUserCheck} width="20px" height="20px" color="inherit" />
      ),
      permissionPrefix: "attempt",
      children: [
        {
          name: "RO Attempt",
          layout: "/officer",
          icon: (
            <Icon
              as={BsFillPeopleFill}
              width="20px"
              height="20px"
              color="inherit"
            />
          ),
          path: "/attempt/consumer",
          permissionPrefix: "consumer",
        },
        {
          name: "DC Attempt",
          layout: "/officer",
          path: "/attempt/dc-form",
          icon: (
            <Icon
              as={BsPersonBadge}
              width="20px"
              height="20px"
              color="inherit"
            />
          ),
          permissionPrefix: "consumer",
        },
      ],
    },
    {
      name: "Location",
      layout: "/officer",
      path: "",
      icon: (
        <Icon as={FaMapMarkedAlt} width="20px" height="20px" color="inherit" />
      ),
      permissionPrefix: "location",
      children: [
        {
          name: "Live Tracking",
          layout: "/officer",
          path: "/location/live",
          icon: (
            <Icon
              as={GiProgression}
              width="20px"
              height="20px"
              color="inherit"
            />
          ),
          permissionPrefix: "location",
        },
        {
          name: "History",
          layout: "/officer",
          icon: (
            <Icon as={FaHistory} width="20px" height="20px" color="inherit" />
          ),
          path: "/location/history",
          permissionPrefix: "location",
        },
      ],
    },
  ];

  switch (role) {
    case "Administrator":
      return adminRoutes.filter((route) =>
        hasPermission(permissions, route.permissionPrefix, "view")
      );
    case "IBC Manager":
      return adminRoutes.filter((route) => {
        route.children = route.children?.filter((child) => {
          return hasPermission(permissions, child.permissionPrefix, "view");
        });
        return hasPermission(permissions, route.permissionPrefix, "view");
      });

    case "Project Admin":
      return adminRoutes.filter((route) =>
        hasPermission(permissions, route.permissionPrefix, "view")
      );

    case "Supervisor":
      return supervisorRoutes.filter((route) =>
        hasPermission(permissions, route.permissionPrefix, "view")
      );

    default:
      return officerRoutes.filter((route) =>
        hasPermission(permissions, route.permissionPrefix, "view")
      );
  }
};

export default routes;
